/**
 * Unified Page Component
 * 
 * A unified component for rendering all types of pages (main, clients, cases)
 * using a consistent block-based content structure.
 * 
 * Features:
 * - Dynamic content loading from API
 * - Fallback content support
 * - Authentication-aware content fetching
 * - Smooth loading transitions
 * - Error handling with user-friendly messages
 * - SEO-friendly structure
 * - Hierarchical page support (parent/child URLs)
 */
import React, { useEffect, useState, useMemo } from 'react';
import { useParams, useLocation, useNavigate } from 'react-router-dom';
import { ContentBlock, renderBlock } from '../components/blocks/BlockRenderer';

import { apiClient } from '../api/client';
import { useAuth } from '../contexts/AuthContext';
import { useLanguage, getBasePath } from '../components/LanguageRouter';
import { getLocalizedContent } from '../utils/languageUtils';
import { usePageLayoutSettings } from '../hooks/usePageLayoutSettings';
import NotFoundPage from './NotFoundPage';
import LayoutDebugger from '../components/LayoutDebugger';
import './Page.css';

// Define interfaces for different page types
interface BasePageData {
  _id?: string;
  identifier: string;
  title: string;
  overview: ContentBlock[] | null;
  published?: boolean;
  updatedAt?: string;
}

interface CasePageData extends BasePageData {
  slug: string;
}

interface MainPageData extends BasePageData {
  identifier: 'main';
}

interface ClientsPageData extends BasePageData {
  identifier: 'clients';
}

interface CasesPageData extends BasePageData {
  identifier: 'cases';
}

interface CustomPageData extends BasePageData {
  // Custom pages can have any identifier
}

type PageData = CasePageData | MainPageData | ClientsPageData | CasesPageData | CustomPageData;

// Helper function to determine if a path segment is likely a page identifier
const isSystemRoute = (segment: string): boolean => {
  return ['clients', 'cases', 'admin', 'en'].includes(segment);
};

// Helper function to determine page type and identifier from URL
const analyzePageFromUrl = (pathname: string) => {
  const basePath = getBasePath(pathname);
  const segments = basePath.split('/').filter(Boolean); // Remove empty segments
  
  // Root page
  if (segments.length === 0) {
    return { pageType: 'main', identifier: 'main', isHierarchical: false };
  }
  
  // Single segment
  if (segments.length === 1) {
    const segment = segments[0];
    
    // System pages
    if (segment === 'clients') return { pageType: 'clients', identifier: 'clients', isHierarchical: false };
    if (segment === 'cases') return { pageType: 'cases', identifier: 'cases', isHierarchical: false };
    
    // Custom single page
    return { pageType: 'customPage', identifier: segment, isHierarchical: false };
  }
  
  // Two segments - these are now hierarchical pages in the unified system
  if (segments.length === 2) {
    const [first, second] = segments;
    
    // All two-segment URLs are now hierarchical pages (parent/child)
    // Legacy case routes will be handled by redirects at the API level
    return { pageType: 'hierarchicalPage', identifier: second, parentIdentifier: first, isHierarchical: true };
  }
  
  // More than 2 segments - not supported yet
  return { pageType: 'unknown', identifier: null, isHierarchical: false };
};

const Page: React.FC = () => {
  // Get all possible URL parameters
  const { 
    caseId, 
    pageId, 
    parentId, 
    childId 
  } = useParams<{ 
    caseId: string; 
    pageId: string; 
    parentId: string; 
    childId: string; 
  }>();
  
  const location = useLocation();
  const navigate = useNavigate();
  const { user } = useAuth();
  const { currentLanguage } = useLanguage();
  
  // Extract version parameter from URL
  const urlParams = new URLSearchParams(location.search);
  const versionId = urlParams.get('version');
  
  // State for the fetched page data, loading status, and errors
  const [pageData, setPageData] = useState<PageData | null>(null);
  const [isLoading, setIsLoading] = useState<boolean>(true);
  const [error, setError] = useState<string | null>(null);
  const [usingFallback, setUsingFallback] = useState<boolean>(false);

  // Analyze the current URL to determine page type and identifier
  const urlAnalysis = useMemo(() => {
    return analyzePageFromUrl(location.pathname);
  }, [location.pathname]);

  // Determine final page type and identifier based on URL analysis and route parameters
  const { pageType, pageIdentifier } = useMemo(() => {
    const { pageType: analyzedType, identifier, parentIdentifier, isHierarchical } = urlAnalysis;
    
    // Handle different route parameter combinations
    if (caseId) {
      // Legacy case route (/cases/:caseId or /clients/:caseId)
      return { pageType: 'case', pageIdentifier: caseId };
    } else if (parentId && childId) {
      // Hierarchical route (/:parentId/:childId)
      return { pageType: 'hierarchicalPage', pageIdentifier: childId };
    } else if (pageId) {
      // Single page route (/:pageId)
      return { pageType: 'customPage', pageIdentifier: pageId };
    } else {
      // Use URL analysis
      return { pageType: analyzedType, pageIdentifier: identifier || 'main' };
    }
  }, [urlAnalysis, caseId, pageId, parentId, childId]);

  if (import.meta.env.DEV) {
    console.log('Page URL Analysis:', {
      pathname: location.pathname,
      basePath: getBasePath(location.pathname),
      urlAnalysis,
      params: { caseId, pageId, parentId, childId },
      finalResult: { pageType, pageIdentifier }
    });
  }

  // Load page-specific layout settings first
  const { loading: layoutLoading } = usePageLayoutSettings(pageIdentifier);

  useEffect(() => {
    const fetchPageData = async () => {
      // Don't start loading page data until layout settings are ready
      if (layoutLoading) {
        return;
      }

      setIsLoading(true);
      setError(null);
      setPageData(null);
      setUsingFallback(false);

      if (import.meta.env.DEV) {
        console.log(`Fetching data for ${pageType} page (${pageIdentifier})${user ? ' (authenticated)' : ' (unauthenticated)'}`);
      }

      try {
        let data: PageData | null = null;

        // Fetch data based on page type
        switch (pageType) {
          case 'main':
            data = await apiClient.getPage('main', currentLanguage, versionId || undefined);
            break;
          case 'clients':
            data = await apiClient.getPage('clients', currentLanguage, versionId || undefined);
            break;
          case 'cases':
            data = await apiClient.getPage('cases', currentLanguage, versionId || undefined);
            break;
          case 'customPage':
            if (!pageIdentifier) throw new Error('No page identifier provided');
            data = await apiClient.getPage(pageIdentifier, currentLanguage, versionId || undefined);
            break;
          case 'hierarchicalPage':
            if (!pageIdentifier) throw new Error('No page identifier provided');
            // For hierarchical pages, use the child identifier to fetch the page
            data = await apiClient.getPage(pageIdentifier, currentLanguage, versionId || undefined);
            break;
          case 'case':
            if (!pageIdentifier) throw new Error('No case identifier provided');
            // Cases are now pages - use pages API directly
            data = await apiClient.getPage(pageIdentifier, currentLanguage, versionId || undefined);
            break;
          default:
            throw new Error(`Unknown page type: ${pageType}`);
        }

        if (!data) {
          throw new Error(`No data available for ${pageType} page`);
        }

        // Check if page is hidden and should redirect
        if ((data as any).status === 'HIDDEN' || (data as any).versionStatus === 'HIDDEN' || (data as any).isHidden) {
          // Try to get redirect URL from the page data first, then from redirects API
          const redirectUrl = (data as any).redirectUrl;
          
          if (redirectUrl) {
            console.log(`Page is hidden, redirecting to: ${redirectUrl}`);
            navigate(redirectUrl, { replace: true });
            return;
          }
          
          // Fallback: try to get redirect URL from redirects API
          try {
            const currentPath = location.pathname;
            const redirect = await apiClient.getRedirect(currentPath);
            if (redirect && redirect.redirect && redirect.redirect.toPath) {
              console.log(`Page is hidden, redirecting via API to: ${redirect.redirect.toPath}`);
              navigate(redirect.redirect.toPath, { replace: true });
              return;
            }
          } catch (error) {
            console.log('No redirect found for hidden page via API');
          }
          
          // If no redirect URL is found, show 404
          console.log('No redirect found for hidden page, showing 404');
          setError('Page not found');
          return;
        }

        // Basic validation of overview data
        if (data.overview && !Array.isArray(data.overview)) {
          if (import.meta.env.DEV) {
            console.warn('Fetched overview data is not an array, attempting to parse or fallback.');
          }
          data.overview = [];
        }

        setPageData(data);
        document.title = `${data.title} - Studio Paulides`;

      } catch (err: any) {
        console.error(`Error fetching ${pageType} page data:`, err);
        
        // Handle specific error cases
        if (err.message?.includes('404') || err.status === 404) {
          // For 404 errors, show appropriate message based on authentication status
          if (user) {
            setError(`${pageType === 'case' ? 'Case' : pageType === 'customPage' ? 'Page' : 'Page'} niet gevonden. Mogelijk is deze verwijderd of heeft de URL een typfout.`);
          } else {
            setError(`${pageType === 'case' ? 'Case' : pageType === 'customPage' ? 'Page' : 'Page'} niet gevonden.`);
          }
        } else {
          setError(`Could not load ${pageType === 'customPage' ? 'custom page' : pageType} page: ${err.message || 'Unknown error'}`);
        }

        // Cases are now pages - no fallback data needed
        // All pages either exist in the database or show 404
      } finally {
        setIsLoading(false);
      }
    };

    fetchPageData();
  }, [pageType, pageIdentifier, user, layoutLoading, currentLanguage, versionId, location.pathname]);

  // Memoize content analysis
  const contentAnalysis = useMemo(() => {
    if (!pageData?.overview) {
      return { hasTitleBlock: false, hasH1InContent: false, needsMainHeading: true };
    }
    
    const hasTitleBlock = pageData.overview.some(block => block.type === 'title');
    const firstTextBlock = pageData.overview.find(block => block.type === 'text') as any;
    const localizedContent = firstTextBlock ? getLocalizedContent(firstTextBlock.content, currentLanguage) : '';
    const hasH1InContent = localizedContent && typeof localizedContent === 'string' && localizedContent.includes('<h1');
    
    return {
      hasTitleBlock,
      hasH1InContent,
      needsMainHeading: !hasTitleBlock && !hasH1InContent
    };
  }, [pageData?.overview, currentLanguage]);

  // Render loading state with proper layout structure
  // Show loading when layout is loading OR when page data is loading
  if (layoutLoading || isLoading) {
    return (
      <main className="main-content-area">
        <LayoutDebugger pageId={pageIdentifier} />
        <article className="content-section">
          {/* Use the same container structure as real content */}
          <div className="content-blocks-container">
            
            {/* Title skeleton */}
            <div className="skeleton-heading" style={{
              height: '3rem',
              backgroundColor: 'var(--color-z-30)',
              borderRadius: '8px',
              width: '80%',
              opacity: '0.5',
              animation: 'pulse 4.5s ease-in-out infinite alternate',
              animationDelay: '0.1s',
              willChange: 'opacity'
            }}></div>
            
            {/* Content blocks skeleton - using the same gap as real content */}
            {[1, 2, 3].map((i) => (
              <div key={i} style={{
                height: '8rem',
                backgroundColor: 'var(--color-z-30)',
                borderRadius: '8px',
                opacity: '0.5',
                animation: 'pulse 4.5s ease-in-out infinite alternate',
                animationDelay: `${i * 0.1}s`,
                willChange: 'opacity'
              }}></div>
            ))}
          </div>
          
          <style>{`
            @keyframes pulse {
              0% { opacity: 0.3; }
              100% { opacity: 0.7; }
            }
            
            @media (prefers-color-scheme: dark) {
              .skeleton-loader div {
                background-color: var(--color-z-90) !important;
                opacity: 0.5 !important;
              }
            }
          `}</style>
        </article>
      </main>
    );
  }

  // Render error state
  if (error) {
    // Check if this is a 404 error
    const is404Error = error.includes('404') || error.includes('niet gevonden') || error.includes('not found');
    
    if (is404Error) {
      return <NotFoundPage />;
    }
    
    return (
      <main className="main-content-area">
        <LayoutDebugger pageId={pageIdentifier} />
        <article 
          className="content-section" 
          style={{ 
            paddingTop: 'var(--spacing-5)',
            paddingLeft: '1rem', 
            paddingRight: '1rem',
            textAlign: 'center'
          }} 
        >
          <div className="error-state">
            <div style={{
              fontSize: '3rem',
              marginBottom: '1rem',
              opacity: 0.5
            }}>⚠️</div>
            <h1 className="main-heading" style={{ marginBottom: '1rem' }}>
              Er ging iets mis
            </h1>
            <p className="section-text" style={{ 
              marginBottom: '2rem',
              color: 'var(--color-z-70)'
            }}>
              We konden de pagina niet laden. Dit kan tijdelijk zijn door een verbindingsprobleem.
            </p>
            <div style={{ marginBottom: '2rem' }}>
              <button 
                onClick={() => window.location.reload()} 
                className="button button--primary"
                style={{ marginRight: '1rem' }}
              >
                Probeer opnieuw
              </button>
              <button 
                onClick={() => window.history.back()} 
                className="button button--secondary"
              >
                Ga terug
              </button>
            </div>
          </div>
        </article>
      </main>
    );
  }

  // Render content
  return (
    <main className="main-content-area">
      <LayoutDebugger pageId={pageIdentifier} />
      <article className="content-section">
        <div className="content-blocks-container">
          {/* Render main heading if needed */}
          {contentAnalysis.needsMainHeading && pageData && (
            <h1 className="main-heading">{pageData.title}</h1>
          )}
          
          {/* Render content blocks */}
          {pageData?.overview?.map((block, index) => (
            <div key={`${block.type}-${index}`}>
              {renderBlock(block)}
            </div>
          ))}
          
          {/* Show fallback notice if using fallback data */}
          {usingFallback && (
            <div style={{
              padding: '1rem',
              backgroundColor: 'var(--color-warning-bg, #fff3cd)',
              border: '1px solid var(--color-warning-border, #ffeaa7)',
              borderRadius: '8px',
              marginTop: '2rem',
              fontSize: '0.9rem',
              color: 'var(--color-warning-text, #856404)'
            }}>
              <strong>⚠️ Fallback Data:</strong> Deze case wordt getoond met lokale fallback data omdat de server data niet beschikbaar is.
            </div>
          )}
        </div>
      </article>
    </main>
  );
};

export default Page; 